{% extends "otree/Base.html" %}


{% block body_main %}
    <div class="container m-5">
        <h2>
            {% block title %}{{ 'Welcome'|gettext }}{% endblock %}
        </h2>
        <div>
            <form method="get">
                {% if invalid_label %}
                    {% comment %}Translators: If the user enters an invalid participant label{% endcomment %}
                    <p class="error text-danger">{{ 'This participant label was not found'|gettext }}</p>
                {% endif %}
                <label for="participant_label">{{ 'Please enter your participant label.'|gettext }}</label>
                <br/>
                <input type="text" id="participant_label"
                       name="participant_label" class="form-control"/>
                <br/>
                {% next_button %}
            </form>

        </div>
    </div>
{% endblock %}